@@ -65,9 +65,12 @@ typedef enum
 } uvm_push_flag_t;
 
 struct uvm_push_crypto_bundle_struct {
-    // Initialization vector used to decrypt the push
+    // Initialization vector used to decrypt the push on the CPU
     UvmCslIv iv;
 
+    // Key version used to decrypt the push on the CPU
+    NvU32 key_version;
+
     // Size of the pushbuffer that is encrypted/decrypted
     NvU32 push_size;
 };
@@ -208,14 +211,13 @@ NV_STATUS __uvm_push_begin_acquire_on_channel_with_info(uvm_channel_t *channel,
                                                         const char *format, ...);
 
 // Internal helper for uvm_push_begin_on_reserved channel
-__attribute__ ((format(printf, 7, 8)))
-NV_STATUS __uvm_push_begin_acquire_on_reserved_channel_with_info(uvm_channel_t *channel,
-                                                                 uvm_tracker_t *tracker,
-                                                                 uvm_push_t *push,
-                                                                 const char *filename,
-                                                                 const char *function,
-                                                                 int line,
-                                                                 const char *format, ...);
+__attribute__ ((format(printf, 6, 7)))
+NV_STATUS __uvm_push_begin_on_reserved_channel_with_info(uvm_channel_t *channel,
+                                                         uvm_push_t *push,
+                                                         const char *filename,
+                                                         const char *function,
+                                                         int line,
+                                                         const char *format, ...);
 // Begin a push on a channel of channel_type type
 // Picks the first available channel. If all channels of the given type are
 // busy, spin waits for one to become available.
@@ -269,8 +271,8 @@ NV_STATUS __uvm_push_begin_acquire_on_reserved_channel_with_info(uvm_channel_t *
 //
 // Locking: on success acquires the concurrent push semaphore until
 //          uvm_push_end()
-#define uvm_push_begin_on_reserved_channel(channel, push, format, ...)              \
-    __uvm_push_begin_acquire_on_reserved_channel_with_info((channel), NULL, (push), \
+#define uvm_push_begin_on_reserved_channel(channel, push, format, ...) \
+    __uvm_push_begin_on_reserved_channel_with_info((channel), (push),  \
         __FILE__, __FUNCTION__, __LINE__, (format), ##__VA_ARGS__)
 
 // Same as uvm_push_begin_on_channel except it also acquires the input tracker
@@ -324,6 +326,11 @@ static void uvm_push_get_tracker_entry(uvm_push_t *push, uvm_tracker_entry_t *en
 // Subsequently pushed GPU work will not start before all the work tracked by
 // tracker is complete.
 // Notably a NULL tracker is handled the same way as an empty tracker.
+//
+// If dependencies across GPUs are not allowed in the current configuration
+// (see uvm_push_allow_dependencies_across_gpus), the caller is responsible for
+// ensuring that the input tracker does not contain dependencies on GPUs other
+// than the one associated with the push.
 void uvm_push_acquire_tracker(uvm_push_t *push, uvm_tracker_t *tracker);
 
 // Set a push flag
@@ -334,6 +341,14 @@ static void uvm_push_set_flag(uvm_push_t *push, uvm_push_flag_t flag)
     __set_bit(flag, push->flags);
 }
 
+// Check if a push flag is set
+static bool uvm_push_test_flag(uvm_push_t *push, uvm_push_flag_t flag)
+{
+    UVM_ASSERT_MSG(flag < UVM_PUSH_FLAG_COUNT, "flag %u\n", (unsigned)flag);
+
+    return test_bit(flag, push->flags);
+}
+
 // Get and reset (if set) a push flag
 static bool uvm_push_get_and_reset_flag(uvm_push_t *push, uvm_push_flag_t flag)
 {
@@ -377,11 +392,6 @@ static bool uvm_push_has_space(uvm_push_t *push, NvU32 free_space)
 NV_STATUS uvm_push_begin_fake(uvm_gpu_t *gpu, uvm_push_t *push);
 void uvm_push_end_fake(uvm_push_t *push);
 
-static bool uvm_push_is_fake(uvm_push_t *push)
-{
-    return !push->channel;
-}
-
 // Begin an inline data fragment in the push
 //
 // The inline data will be ignored by the GPU, but can be referenced from
@@ -477,4 +487,8 @@ static uvm_push_info_t *uvm_push_info_from_push(uvm_push_t *push)
     return &channel->push_infos[push->push_info_index];
 }
 
+// Returns true if a push is allowed to depend on pushes on other GPUs: work
+// dependencies across GPUs are permitted.
+bool uvm_push_allow_dependencies_across_gpus(void);
+
 #endif // __UVM_PUSH_H__
