@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -256,7 +256,7 @@ GspMsgQueuesInit
 
     memdescSetFlag(pMQCollection->pSharedMemDesc, MEMDESC_FLAGS_KERNEL_MODE, NV_TRUE);
 
-    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_58, 
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_58,
                     pMQCollection->pSharedMemDesc);
     NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus, error_ret);
 
@@ -524,12 +524,12 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
         portMemCopy((NvU8*)pCQE->aadBuffer, sizeof(pCQE->aadBuffer), (NvU8 *)&pCQE->seqNum, sizeof(pCQE->seqNum));
 
         // We need to encrypt the full queue elements to obscure the data.
-        nvStatus = ccslEncrypt(pCC->pRpcCcslCtx,
+        nvStatus = ccslEncryptWithRotationChecks(pCC->pRpcCcslCtx,
                                (pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
-                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
+                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                (NvU8*)pCQE->aadBuffer,
                                sizeof(pCQE->aadBuffer),
-                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
+                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                pCQE->authTagBuffer);
 
         if (nvStatus != NV_OK)
@@ -772,13 +772,13 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
     if (gpuIsCCFeatureEnabled(pGpu))
     {
         ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
-        nvStatus = ccslDecrypt(pCC->pRpcCcslCtx,
+        nvStatus = ccslDecryptWithRotationChecks(pCC->pRpcCcslCtx,
                                (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
-                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
+                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                NULL,
                                (NvU8*)pMQI->pCmdQueueElement->aadBuffer,
                                sizeof(pMQI->pCmdQueueElement->aadBuffer),
-                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
+                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                ((NvU8*)pMQI->pCmdQueueElement->authTagBuffer));
 
         if (nvStatus != NV_OK)
