@@ -1,4 +1,4 @@
-/*
+ /*
  * SPDX-FileCopyrightText: Copyright (c) 2016-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
@@ -193,7 +193,7 @@ struct RM_POOL_ALLOC_MEM_RESERVE_INFO
  *        pool.
  *
  * @param[in] pCtx     Context for upstream allocator.
- * @param[in] pageSize Only for debugging.
+ * @param[in] pageSize Page size to use when allocating from PMA
  * @param[in] pPage    Output page handle from upstream.
  *
  * @return NV_STATUS
@@ -203,20 +203,20 @@ allocUpstreamTopPool
 (
     void             *pCtx,
     NvU64             pageSize,
+    NvU64             numPages, 
     POOLALLOC_HANDLE *pPage
 )
 {
     PMA_ALLOCATION_OPTIONS      allocOptions = {0};
     RM_POOL_ALLOC_MEM_RESERVE_INFO *pMemReserveInfo;
-    NV_STATUS                   status;
+    NvU64 i, pageBegin;
+    NV_STATUS status;
 
     NV_ASSERT_OR_RETURN(NULL != pCtx, NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN(NULL != pPage, NV_ERR_INVALID_ARGUMENT);
 
-    // TODO: Replace the direct call to PMA with function pointer.
     pMemReserveInfo = (RM_POOL_ALLOC_MEM_RESERVE_INFO *)pCtx;
-    allocOptions.flags = PMA_ALLOCATE_PINNED | PMA_ALLOCATE_PERSISTENT |
-                         PMA_ALLOCATE_CONTIGUOUS;
+    allocOptions.flags = PMA_ALLOCATE_PINNED | PMA_ALLOCATE_PERSISTENT;
 
     if (pMemReserveInfo->bSkipScrub)
     {
@@ -228,18 +228,54 @@ allocUpstreamTopPool
         allocOptions.flags |= PMA_ALLOCATE_PROTECTED_REGION;
     }
 
-    status = pmaAllocatePages(pMemReserveInfo->pPma,
-                              pMemReserveInfo->pmaChunkSize/PMA_CHUNK_SIZE_64K,
-                              PMA_CHUNK_SIZE_64K,
-                              &allocOptions,
-                              &pPage->address);
-    if (status != NV_OK)
+    //
+    // Some tests fail page table and directory allocation when close to all FB is allocated if we allocate contiguously.
+    // For now, we're special-casing this supported pageSize and allocating the 64K pages discontigously.
+    // TODO: Unify the codepaths so that all pages allocated discontiguously (not currently supported by PMA)
+    //
+    if (pageSize == PMA_CHUNK_SIZE_64K)
     {
+        NvU64 *pPageStore = portMemAllocNonPaged(sizeof(NvU64) * numPages);
+        NV_STATUS status = NV_OK;
+        NV_ASSERT_OK_OR_GOTO(status,
+            pmaAllocatePages(pMemReserveInfo->pPma,
+                numPages,
+                pageSize,
+                &allocOptions,
+                pPageStore),
+            free_mem);
+
+        for (i = 0; i < numPages; i++)
+        {
+            pPage[i].address = pPageStore[i];
+            pPage[i].pMetadata = NULL;
+        }
+free_mem:
+        portMemFree(pPageStore);
         return status;
     }
 
-    pPage->pMetadata = NULL;
+    allocOptions.flags |= PMA_ALLOCATE_CONTIGUOUS;
 
+    for (i = 0; i < numPages; i++)
+    {
+        NV_ASSERT_OK_OR_GOTO(status, pmaAllocatePages(pMemReserveInfo->pPma,
+            pageSize / PMA_CHUNK_SIZE_64K,
+            PMA_CHUNK_SIZE_64K,
+            &allocOptions,
+            &pageBegin), err);
+        pPage[i].address = pageBegin;
+        pPage[i].pMetadata = NULL;
+    }
+
+    return NV_OK;
+err:
+    for (;i > 0; i--)
+    {
+        NvU32 flags = pMemReserveInfo->bSkipScrub ? PMA_FREE_SKIP_SCRUB : 0;
+        pmaFreePages(pMemReserveInfo->pPma, &pPage[i - 1].address, 1,
+            pageSize, flags);
+    }
     return status;
 }
 
@@ -258,17 +294,28 @@ allocUpstreamLowerPools
 (
     void             *pCtx,
     NvU64             pageSize,
+    NvU64             numPages,
     POOLALLOC_HANDLE *pPage
 )
 {
     NV_STATUS status;
+    NvU64 i;
 
     NV_ASSERT_OR_RETURN(NULL != pCtx, NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN(NULL != pPage, NV_ERR_INVALID_ARGUMENT);
 
-    status = poolAllocate((POOLALLOC *)pCtx, pPage);
-    NV_ASSERT_OR_RETURN(status == NV_OK, status);
-
+    for(i = 0; i < numPages; i++)
+    {
+        NV_ASSERT_OK_OR_GOTO(status,
+            poolAllocate((POOLALLOC *)pCtx, &pPage[i]),
+            cleanup);
+    }
+    return NV_OK;
+cleanup:
+    for(;i > 0; i--)
+    {
+        poolFree((POOLALLOC *)pCtx, &pPage[i-1]);
+    }
     return status;
 }
 
@@ -441,7 +488,7 @@ rmMemPoolSetup
     // of reserving memory for page tables required for mapping GR context buffers
     // in the channel vaspace. See bug 200590870 and 200614517.
     //
-    if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM)
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS)
     {
         flags = FLD_SET_DRF(_RMPOOL, _FLAGS, _AUTO_POPULATE, _ENABLE, flags);
     }
